# -*- coding: utf-8 -*-
from django.conf import settings

from appconf import AppConf


class AldrynSearchAppConf(AppConf):

    CMS_PAGE = True
    DEFAULT_LANGUAGE = settings.LANGUAGE_CODE
    INDEX_BASE_CLASS = 'aldryn_search.base.AldrynIndexBase'
    LANGUAGE_FROM_ALIAS = 'aldryn_search.utils.language_from_alias'
    PAGINATION = 10
    REGISTER_APPHOOK = True

    class Meta:
        prefix = 'ALDRYN_SEARCH'



# -*- coding: utf-8 -*-
from django.db.models import Q
from django.utils import timezone

from cms.models import CMSPlugin, Title

from .compat import GTE_CMS_35
from .conf import settings
from .helpers import get_plugin_index_data
from .utils import clean_join, get_index_base, strip_tags


# Backwards compatibility
_strip_tags = strip_tags


class TitleIndex(get_index_base()):
    index_title = True

    object_actions = ('publish', 'unpublish')
    haystack_use_for_indexing = settings.ALDRYN_SEARCH_CMS_PAGE

    def prepare_pub_date(self, obj):
        return obj.page.publication_date

    def prepare_login_required(self, obj):
        return obj.page.login_required

    def prepare_site_id(self, obj):
        if not GTE_CMS_35:
            return obj.page.site_id
        return obj.page.node.site_id

    def get_language(self, obj):
        return obj.language

    def get_url(self, obj):
        return obj.page.get_absolute_url()

    def get_title(self, obj):
        return obj.title

    def get_description(self, obj):
        return obj.meta_description or None

    def get_plugin_queryset(self, language):
        queryset = CMSPlugin.objects.filter(language=language)
        return queryset

    def get_page_placeholders(self, page):
        """
        In the project settings set up the variable

        PLACEHOLDERS_SEARCH_LIST = {
            # '*' is mandatory if you define at least one slot rule
            '*': {
                'include': [ 'slot1', 'slot2', etc. ],
                'exclude': [ 'slot3', 'slot4', etc. ],
            }
            'reverse_id_alpha': {
                'include': [ 'slot1', 'slot2', etc. ],
                'exclude': [ 'slot3', 'slot4', etc. ],
            },
            'reverse_id_beta': {
                'include': [ 'slot1', 'slot2', etc. ],
                'exclude': [ 'slot3', 'slot4', etc. ],
            },
            'reverse_id_only_include': {
                'include': [ 'slot1', 'slot2', etc. ],
            },
            'reverse_id_only_exclude': {
                'exclude': [ 'slot3', 'slot4', etc. ],
            },
            # exclude it from the placehoders search list
            # (however better to remove at all to exclude it)
            'reverse_id_empty': []
            etc.
        }

        or leave it empty

        PLACEHOLDERS_SEARCH_LIST = {}
        """
        reverse_id = page.reverse_id
        args = []
        kwargs = {}

        placeholders_by_page = getattr(settings, 'PLACEHOLDERS_SEARCH_LIST', {})

        if placeholders_by_page:
            filter_target = None
            excluded = []
            slots = []
            if '*' in placeholders_by_page:
                filter_target = '*'
            if reverse_id and reverse_id in placeholders_by_page:
                filter_target = reverse_id
            if not filter_target:
                raise AttributeError('Leave PLACEHOLDERS_SEARCH_LIST empty or set up at least the generic handling')
            if 'include' in placeholders_by_page[filter_target]:
                slots = placeholders_by_page[filter_target]['include']
            if 'exclude' in placeholders_by_page[filter_target]:
                excluded = placeholders_by_page[filter_target]['exclude']
            diff = set(slots) - set(excluded)
            if diff:
                kwargs['slot__in'] = diff
            else:
                args.append(~Q(slot__in=excluded))
        return page.placeholders.filter(*args, **kwargs)

    def get_search_data(self, obj, language, request):
        current_page = obj.page
        placeholders = self.get_page_placeholders(current_page)
        plugins = self.get_plugin_queryset(language).filter(placeholder__in=placeholders)
        text_bits = []

        for base_plugin in plugins:
            plugin_text_content = self.get_plugin_search_text(base_plugin, request)
            text_bits.append(plugin_text_content)

        page_meta_description = current_page.get_meta_description(fallback=False, language=language)

        if page_meta_description:
            text_bits.append(page_meta_description)

        page_meta_keywords = getattr(current_page, 'get_meta_keywords', None)

        if callable(page_meta_keywords):
            text_bits.append(page_meta_keywords())

        return clean_join(' ', text_bits)

    def get_plugin_search_text(self, base_plugin, request):
        plugin_content_bits = get_plugin_index_data(base_plugin, request)
        return clean_join(' ', plugin_content_bits)

    def get_model(self):
        return Title

    def get_index_queryset(self, language):
        queryset = Title.objects.public().filter(
            Q(page__publication_date__lt=timezone.now()) | Q(page__publication_date__isnull=True),
            Q(page__publication_end_date__gte=timezone.now()) | Q(page__publication_end_date__isnull=True),
            Q(redirect__exact='') | Q(redirect__isnull=True),
            language=language
        ).select_related('page')
        if GTE_CMS_35:
            queryset = queryset.select_related('page__node')
        return queryset.distinct()

    def should_update(self, instance, **kwargs):
        # We use the action flag to prevent
        # updating the cms page on save.
        return kwargs.get('object_action') in self.object_actions



from django.utils.translation import ugettext_lazy as _

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool

from .conf import settings


class AldrynSearchApphook(CMSApp):
    name = _("aldryn search")

    def get_urls(self, *args, **kwargs):
        return ['aldryn_search.urls']


if settings.ALDRYN_SEARCH_REGISTER_APPHOOK:
    apphook_pool.register(AldrynSearchApphook)



# -*- coding: utf-8 -*-
'''
Created on Nov 30, 2015
@author: jakob
'''
from django.dispatch.dispatcher import receiver

from cms.models.titlemodels import Title
from cms.signals import post_publish, post_unpublish

from .signals import add_to_index, remove_from_index


@receiver(post_publish, dispatch_uid='publish_cms_page')
def publish_cms_page(sender, instance, language, **kwargs):
    title = instance.publisher_public.get_title_obj(language)
    add_to_index.send(sender=Title, instance=title, object_action='publish')


@receiver(post_unpublish, dispatch_uid='unpublish_cms_page')
def unpublish_cms_page(sender, instance, language, **kwargs):
    title = instance.publisher_public.get_title_obj(language)
    remove_from_index.send(sender=Title, instance=title, object_action='unpublish')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re
import six

from lxml.html.clean import Cleaner as LxmlCleaner
from lxml.etree import ParseError, ParserError


from django.core.exceptions import ImproperlyConfigured
from django.db import models
try:
    from django.utils.encoding import force_unicode
except ImportError:
    from django.utils.encoding import force_text as force_unicode
try:
    import importlib
except ImportError:
    # python 2.6 compatibility
    from django.utils import importlib

from haystack import DEFAULT_ALIAS
from haystack.indexes import SearchIndex

from cms.utils.i18n import get_language_code

from .conf import settings


def alias_from_language(language):
    """
    Returns alias if alias is a valid language.
    """
    language = get_language_code(language)

    if language == settings.ALDRYN_SEARCH_DEFAULT_LANGUAGE:
        return DEFAULT_ALIAS
    return language


def clean_join(separator, iterable):
    """
    Filters out iterable to only join non empty items.
    """
    return separator.join(filter(None, iterable))


def get_callable(string_or_callable):
    """
    If given a callable then it returns it, otherwise it resolves the path
    and returns an object.
    """
    if callable(string_or_callable):
        return string_or_callable
    else:
        module_name, object_name = string_or_callable.rsplit('.', 1)
        module = importlib.import_module(module_name)
        return getattr(module, object_name)


def _get_language_from_alias_func():
    path_or_callable = settings.ALDRYN_SEARCH_LANGUAGE_FROM_ALIAS

    if path_or_callable:
        try:
            func = get_callable(path_or_callable)
        except AttributeError as error:
            raise ImproperlyConfigured('ALDRYN_SEARCH_LANGUAGE_FROM_ALIAS: %s' % (str(error)))
        if not callable(func):
            raise ImproperlyConfigured('ALDRYN_SEARCH_LANGUAGE_FROM_ALIAS: %s is not callable' % func)
    else:
        func = None
    return func


def get_index_base():
    index_string = settings.ALDRYN_SEARCH_INDEX_BASE_CLASS
    try:
        BaseClass = get_callable(index_string)
    except AttributeError as error:
        raise ImproperlyConfigured('ALDRYN_SEARCH_INDEX_BASE_CLASS: %s' % (str(error)))

    if not issubclass(BaseClass, SearchIndex):
        exception_message = ('ALDRYN_SEARCH_INDEX_BASE_CLASS: %s is not a '
                             'subclass of haystack.indexes.SearchIndex')
        raise ImproperlyConfigured(exception_message % index_string)

    required_fields = ['text', 'language']

    if not all(field in BaseClass.fields for field in required_fields):
        exception_message = ('ALDRYN_SEARCH_INDEX_BASE_CLASS: %s must contain '
                             'at least these fields: %s')
        raise ImproperlyConfigured(exception_message % (index_string,
                                                        required_fields))
    return BaseClass


def language_from_alias(alias):
    """
    Returns alias if alias is a valid language.
    """
    languages = [language[0] for language in settings.LANGUAGES]

    return alias if alias in languages else None


def get_field_value(obj, name):
    """
    Given a model instance and a field name (or attribute),
    returns the value of the field or an empty string.
    """
    fields = name.split('__')

    name = fields[0]

    try:
        obj._meta.get_field(name)
    except (AttributeError, models.FieldDoesNotExist):
        # we catch attribute error because obj will not always be a model
        # specially when going through multiple relationships.
        value = getattr(obj, name, None) or ''
    else:
        value = getattr(obj, name)

    if len(fields) > 1:
        remaining = '__'.join(fields[1:])
        return get_field_value(value, remaining)
    return value


def get_model_path(model_or_string):
    if not isinstance(model_or_string, six.string_types):
        # it's a model class
        app_label = model_or_string._meta.app_label
        model_name = model_or_string._meta.object_name
        model_or_string = '{0}.{1}'.format(app_label, model_name)
    return model_or_string.lower()


def _strip_tags(value):
    """
    Returns the given HTML with all tags stripped.
    This is a copy of django.utils.html.strip_tags, except that it adds some
    whitespace in between replaced tags to make sure words are not erroneously
    concatenated.
    """
    return re.sub(r'<[^>]*?>', ' ', force_unicode(value))


def strip_tags(value):
    """
    Returns the given HTML with all tags stripped.
    We use lxml to strip all js tags and then hand the result to django's
    strip tags. If value isn't valid, just return value since there is
    no tags to strip.
    """
    if isinstance(value, six.string_types):
        value = value.strip()

        try:
            partial_strip = LxmlCleaner().clean_html(value)
        except (ParseError, ParserError):
            # Error could occur because of invalid html document,
            # including '' values. We don't want to return empty handed.
            partial_strip = value
        value = _strip_tags(partial_strip)
        return value.strip()  # clean cases we have <div>\n\n</div>
    return value



from cms.api import create_page, add_plugin
from cms.models import CMSPlugin, Title
from cms.models.placeholdermodel import Placeholder
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.template import engines
from django.test import TestCase
from haystack import connections
from haystack.constants import DEFAULT_ALIAS
from haystack.query import SearchQuerySet

from aldryn_search.search_indexes import TitleIndex

from .helpers import get_plugin_index_data, get_request


def template_from_string(value):
    """Create an engine-specific template based on provided string.
    """
    return engines.all()[0].from_string(value)


class FakeTemplateLoader(object):
    is_usable = True

    def __init__(self, name, dirs):
        pass

    def __iter__(self):
        yield self.__class__
        yield "{{baz}}"


class NotIndexedPlugin(CMSPluginBase):
    model = CMSPlugin
    plugin_content = 'rendered plugin content'
    render_template = template_from_string(plugin_content)

    def render(self, context, instance, placeholder):
        return context

plugin_pool.register_plugin(NotIndexedPlugin)


class HiddenPlugin(CMSPluginBase):
    model = CMSPlugin
    plugin_content = 'never search for this content'
    render_template = template_from_string(plugin_content)

    def render(self, context, instance, placeholder):
        return context

plugin_pool.register_plugin(HiddenPlugin)


class BaseTestCase(TestCase):
    def setUp(self):
        pass

    def get_title_index(self):
        search_conn = connections[DEFAULT_ALIAS]
        unified_index = search_conn.get_unified_index()
        index = unified_index.get_index(Title)
        return index


class PluginIndexingTests(BaseTestCase):

    def setUp(self):
        self.index = TitleIndex()
        self.request = get_request(language='en')

    def get_plugin(self):
        instance = CMSPlugin(
            language='en',
            plugin_type="NotIndexedPlugin",
            placeholder=Placeholder(id=1235)
        )
        instance.cmsplugin_ptr = instance
        instance.pk = 1234  # otherwise plugin_meta_context_processor() crashes
        return instance

    def test_plugin_indexing_is_enabled_by_default(self):
        cms_plugin = self.get_plugin()
        indexed_content = self.index.get_plugin_search_text(cms_plugin, self.request)
        self.assertEqual(NotIndexedPlugin.plugin_content, indexed_content)

    def test_plugin_indexing_can_be_disabled_on_model(self):
        cms_plugin = self.get_plugin()
        cms_plugin.search_fulltext = False
        indexed_content = self.index.get_plugin_search_text(cms_plugin, self.request)
        self.assertEqual('', indexed_content)

    def test_plugin_indexing_can_be_disabled_on_plugin(self):
        NotIndexedPlugin.search_fulltext = False

        try:
            self.assertEqual('', self.index.get_plugin_search_text(self.get_plugin(), self.request))
        finally:
            del NotIndexedPlugin.search_fulltext

    def test_page_title_is_indexed_using_prepare(self):
        """This tests the indexing path way used by update_index mgmt command"""
        page = create_page(title="home", template="page.html", language="en")
        index = self.get_title_index()

        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.index_queryset(DEFAULT_ALIAS)  # initialises index._backend_alias
        indexed = index.prepare(title)
        self.assertEqual('home', indexed['title'])
        self.assertEqual('home', indexed['text'])

    def test_page_title_is_indexed_using_update_object(self):
        """This tests the indexing path way used by the RealTimeSignalProcessor"""
        page = create_page(title="home", template="page.html", language="en")
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.update_object(title, using=DEFAULT_ALIAS)
        indexed = index.prepared_data
        self.assertEqual('home', indexed['title'])
        self.assertEqual('home', indexed['text'])


class PluginFilterIndexingTests(BaseTestCase):

    def test_page_title_is_indexed_using_prepare_with_filter_option(self):
        """This tests the indexing path way used by update_index mgmt command"""
        page = create_page(title="test_page", reverse_id='testpage', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.index_queryset(DEFAULT_ALIAS)  # initialises index._backend_alias
        indexed = index.prepare(title)
        self.assertEqual('test_page', indexed['title'])
        self.assertEqual('test_page rendered plugin content', indexed['text'])

    def test_page_title_is_indexed_using_update_object_with_filter_option(self):
        """This tests the indexing path way used by the RealTimeSignalProcessor"""
        page = create_page(title="test_page", reverse_id='testpage', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.update_object(title, using=DEFAULT_ALIAS)
        indexed = index.prepared_data
        self.assertEqual('test_page', indexed['title'])
        self.assertEqual('test_page rendered plugin content', indexed['text'])


class PluginExcludeAndFilterIndexingTests2(BaseTestCase):

    def test_page_title_is_indexed_using_prepare_with_excluding_filter_option2(self):
        """This tests the indexing path way used by update_index mgmt command"""
        page = create_page(title="test_page2", reverse_id='testpage2', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.index_queryset(DEFAULT_ALIAS)  # initialises index._backend_alias
        indexed = index.prepare(title)
        self.assertEqual('test_page2', indexed['title'])
        self.assertEqual('test_page2 rendered plugin content never search for this content', indexed['text'])

    def test_page_title_is_indexed_using_update_object_with_excluding_filter_option2(self):
        """This tests the indexing path way used by the RealTimeSignalProcessor"""
        page = create_page(title="test_page2", reverse_id='testpage2', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.update_object(title, using=DEFAULT_ALIAS)
        indexed = index.prepared_data
        self.assertEqual('test_page2', indexed['title'])
        self.assertEqual('test_page2 rendered plugin content never search for this content', indexed['text'])


class PluginExcludeAndFilterIndexingTests3(BaseTestCase):

    def test_page_title_is_indexed_using_prepare_with_excluding_filter_option3(self):
        """This tests the indexing path way used by update_index mgmt command"""
        page = create_page(title="test_page3", reverse_id='testpage3', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.index_queryset(DEFAULT_ALIAS)  # initialises index._backend_alias
        indexed = index.prepare(title)
        self.assertEqual('test_page3', indexed['title'])
        self.assertEqual('test_page3', indexed['text'])

    def test_page_title_is_indexed_using_update_object_with_excluding_filter_option3(self):
        """This tests the indexing path way used by the RealTimeSignalProcessor"""
        page = create_page(title="test_page3", reverse_id='testpage3', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.update_object(title, using=DEFAULT_ALIAS)
        indexed = index.prepared_data
        self.assertEqual('test_page3', indexed['title'])
        self.assertEqual('test_page3', indexed['text'])


class PluginExcludeAndFilterIndexingTests4(BaseTestCase):

    def test_page_title_is_indexed_using_prepare_with_excluding_filter_option4(self):
        """This tests the indexing path way used by update_index mgmt command"""
        page = create_page(title="test_page4", reverse_id='testpage4', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.index_queryset(DEFAULT_ALIAS)  # initialises index._backend_alias
        indexed = index.prepare(title)
        self.assertEqual('test_page4', indexed['title'])
        self.assertEqual('test_page4 rendered plugin content', indexed['text'])

    def test_page_title_is_indexed_using_update_object_with_excluding_filter_option4(self):
        """This tests the indexing path way used by the RealTimeSignalProcessor"""
        page = create_page(title="test_page4", reverse_id='testpage4', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.update_object(title, using=DEFAULT_ALIAS)
        indexed = index.prepared_data
        self.assertEqual('test_page4', indexed['title'])
        self.assertEqual('test_page4 rendered plugin content', indexed['text'])


class PluginExcludeAndFilterIndexingTests5(BaseTestCase):

    def test_page_title_is_indexed_using_prepare_with_excluding_filter_option5(self):
        """This tests the indexing path way used by update_index mgmt command"""
        page = create_page(title="test_page5", reverse_id='testpage5', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.index_queryset(DEFAULT_ALIAS)  # initialises index._backend_alias
        indexed = index.prepare(title)
        self.assertEqual('test_page5', indexed['title'])
        self.assertEqual('test_page5 never search for this content', indexed['text'])

    def test_page_title_is_indexed_using_update_object_with_excluding_filter_option5(self):
        """This tests the indexing path way used by the RealTimeSignalProcessor"""
        page = create_page(title="test_page5", reverse_id='testpage5', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.update_object(title, using=DEFAULT_ALIAS)
        indexed = index.prepared_data
        self.assertEqual('test_page5', indexed['title'])
        self.assertEqual('test_page5 never search for this content', indexed['text'])


class PluginExcludeAndFilterIndexingTests6(BaseTestCase):

    def test_page_title_is_indexed_using_prepare_with_excluding_filter_option6(self):
        """This tests the indexing path way used by update_index mgmt command"""
        page = create_page(title="test_page6", reverse_id='testpage6', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.index_queryset(DEFAULT_ALIAS)  # initialises index._backend_alias
        indexed = index.prepare(title)
        self.assertEqual('test_page6', indexed['title'])
        self.assertEqual('test_page6 rendered plugin content never search for this content', indexed['text'])

    def test_page_title_is_indexed_using_update_object_with_excluding_filter_option6(self):
        """This tests the indexing path way used by the RealTimeSignalProcessor"""
        page = create_page(title="test_page6", reverse_id='testpage6', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.update_object(title, using=DEFAULT_ALIAS)
        indexed = index.prepared_data
        self.assertEqual('test_page6', indexed['title'])
        self.assertEqual('test_page6 rendered plugin content never search for this content', indexed['text'])


class PluginExcludeAndFilterIndexingTests7(BaseTestCase):

    def test_page_title_is_indexed_using_prepare_with_excluding_filter_option7(self):
        """This tests the indexing path way used by update_index mgmt command"""
        page = create_page(title="test_page7", reverse_id='testpage7', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.index_queryset(DEFAULT_ALIAS)  # initialises index._backend_alias
        indexed = index.prepare(title)
        self.assertEqual('test_page7', indexed['title'])
        self.assertEqual('test_page7 never search for this content', indexed['text'])

    def test_page_title_is_indexed_using_update_object_with_excluding_filter_option7(self):
        """This tests the indexing path way used by the RealTimeSignalProcessor"""
        page = create_page(title="test_page7", reverse_id='testpage7', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.update_object(title, using=DEFAULT_ALIAS)
        indexed = index.prepared_data
        self.assertEqual('test_page7', indexed['title'])
        self.assertEqual('test_page7 never search for this content', indexed['text'])

class PluginExcludeAndFilterIndexingTests8(BaseTestCase):

    def test_page_title_is_indexed_using_prepare_with_excluding_filter_option8(self):
        """This tests the indexing path way used by update_index mgmt command"""
        page = create_page(title="test_page8", reverse_id='testpage8', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.index_queryset(DEFAULT_ALIAS)  # initialises index._backend_alias
        indexed = index.prepare(title)
        self.assertEqual('test_page8', indexed['title'])
        self.assertEqual('test_page8 rendered plugin content never search for this content', indexed['text'])

    def test_page_title_is_indexed_using_update_object_with_excluding_filter_option8(self):
        """This tests the indexing path way used by the RealTimeSignalProcessor"""
        page = create_page(title="test_page8", reverse_id='testpage8', template="test.html", language="en")
        plugin = add_plugin(page.placeholders.get(slot='content'), NotIndexedPlugin, 'en')
        plugin2 = add_plugin(page.placeholders.get(slot='hidden_content'), HiddenPlugin, 'en')
        index = self.get_title_index()
        title = Title.objects.get(pk=page.title_set.all()[0].pk)
        index.update_object(title, using=DEFAULT_ALIAS)
        indexed = index.prepared_data
        self.assertEqual('test_page8', indexed['title'])
        self.assertEqual('test_page8 rendered plugin content never search for this content', indexed['text'])

# This test case is suitable only if you have a running solr, so if you do
# please uncomment it and ensure that aldryn-search works as expected.
# class UnpublishTest(BaseTestCase):
#
#     def test_unpublish_page(self):
#         page = create_page('test page', 'test.html', 'en', published=True)
#         title = page.publisher_public.get_title_obj('en')
#         self.assertEqual(1, SearchQuerySet().models(Title).filter(id=title.pk).count())
#         page.unpublish('en')
#         self.assertEqual(0, SearchQuerySet().models(Title).filter(id=title.pk).count())



# -*- coding: utf-8 -*-
from django.conf import settings
from django.utils.translation import get_language

from haystack import routers
from haystack.constants import DEFAULT_ALIAS

from .utils import alias_from_language


class LanguageRouter(routers.BaseRouter):

    def for_read(self, **hints):
        language = get_language()
        alias = alias_from_language(language)

        if alias not in settings.HAYSTACK_CONNECTIONS:
            return DEFAULT_ALIAS
        return alias

    def for_write(self, **hints):
        language = get_language()
        alias = alias_from_language(language)

        if alias not in settings.HAYSTACK_CONNECTIONS:
            return DEFAULT_ALIAS
        return alias



# -*- coding: utf-8 -*-
import warnings
from django.utils.translation import override

from haystack import indexes

from .conf import settings
from .helpers import get_request
from .utils import clean_join, _get_language_from_alias_func


language_from_alias = _get_language_from_alias_func()


class AbstractIndex(indexes.SearchIndex):
    text = indexes.CharField(document=True, use_template=False)

    def _get_backend(self, using):
        """
        We set the backend to allow easy access for things like document search.
        """
        self._backend = super(AbstractIndex, self)._get_backend(using)
        self._backend_alias = using
        return self._backend

    def index_queryset(self, using=None):
        self._get_backend(using)
        language = self.get_current_language(using)
        filter_kwargs = self.get_index_kwargs(language)
        qs = self.get_index_queryset(language)
        return (qs.filter(**filter_kwargs))

    def get_index_queryset(self, language):
        return self.get_model().objects.all()

    def prepare(self, obj):
        current_language = self.get_current_language(using=self._backend_alias, obj=obj)

        with override(current_language):
            request = self.get_request_instance(obj, current_language)
            self.prepared_data = super(AbstractIndex, self).prepare(obj)
            self.prepared_data['text'] = self.get_search_data(obj, current_language, request)
            self.prepare_fields(obj, current_language, request)
            return self.prepared_data

    def get_request_instance(self, obj, language):
        return get_request(language)

    def get_language(self, obj):
        """
        Equivalent to self.prepare_language.
        """
        return None

    def get_current_language(self, using=None, obj=None):
        """
        Helper method bound to ALWAYS return a language.

        When obj is not None, this calls self.get_language to try and get a
        language from obj, this is useful when the object itself defines it's
        language in a "language" field.

        If no language was found or obj is None, then we call
        self.get_default_language to try and get a fallback language.
        """
        language = self.get_language(obj) if obj else None
        return language or self.get_default_language(using)

    def get_default_language(self, using):
        """
        When using multiple languages, this allows us to specify a fallback based on the
        backend being used.
        """
        language = None

        if using and language_from_alias:
            language = language_from_alias(using)
        return language or settings.ALDRYN_SEARCH_DEFAULT_LANGUAGE

    def get_index_kwargs(self, language):
        """
        This is called to filter the index queryset.
        """
        return {}

    def get_search_data(self, obj, language, request):
        """
        Returns a string that will be used to populate the text field (primary field).
        """
        return ''

    def prepare_fields(self, obj, language, request):
        """
        This is called to prepare any extra fields.
        """
        pass


class AldrynIndexBase(AbstractIndex):
    # For some apps it makes sense to turn on the title indexing.
    index_title = False

    language = indexes.CharField()
    description = indexes.CharField(indexed=False, stored=True, null=True)
    pub_date = indexes.DateTimeField(null=True)
    login_required = indexes.BooleanField(default=False)
    url = indexes.CharField(stored=True, indexed=False)
    title = indexes.CharField(stored=True, indexed=False)
    site_id = indexes.IntegerField(stored=True, indexed=True, null=True)

    def __init__(self):
        if hasattr(self, 'INDEX_TITLE'):
            warning_message = ('AldrynIndexBase.INDEX_TITLE is deprecated; '
                               'use AldrynIndexBase.index_title instead')
            warnings.warn(warning_message, PendingDeprecationWarning)
        super(AldrynIndexBase, self).__init__()

    def get_url(self, obj):
        """
        Equivalent to self.prepare_url.
        """
        return obj.get_absolute_url()

    def get_title(self, obj):
        """
        Equivalent to self.prepare_title.
        """
        return None

    def get_description(self, obj):
        """
        Equivalent to self.prepare_description.
        """
        return None

    def prepare_fields(self, obj, language, request):
        self.prepared_data['language'] = language
        # We set the following fields here because on some models,
        # the value of these fields is dependent on the active language
        # this being the case we extrapolate the language hacks.
        self.prepared_data['url'] = self.get_url(obj)
        self.prepared_data['title'] = self.get_title(obj)
        self.prepared_data['description'] = self.get_description(obj)

        if self.index_title or getattr(self, 'INDEX_TITLE', False):
            prepared_text = self.prepared_data['text']
            prepared_title = self.prepared_data['title']
            self.prepared_data['text'] = clean_join(
                ' ', [prepared_title, prepared_text]
            )



package_name = 'aldryn_search'
name = 'aldryn-search'
author = 'Benjamin Wohlwend'
author_email = 'piquadrat@gmail.com'
description = "An extension to django CMS to provide multilingual Haystack indexes"
version = __import__(package_name).__version__
project_url = 'http://github.com/aldryn/%s' % name
license = 'BSD'
